#include "scriptinterface.h"

#include "Uni_AllyMeetingText.h"
#include "Uni_AllyMeetingMaps.h"

int32 EnemyCaptured = 0;
int32 MissionAct = 0;
int32 Losses = 0;
bool DiplomatDead = false;
bool PlayerDead = false;
bool Reinforced = false;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam,
	kAlliedTeam,
	kEnemyReinforcementTeam,
	kAlliedReinforcementTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAlliedBorder | kEnemyBorder;
	Specs->fPlayerRace = kNonPirateMission;
	Specs->fEnemyRace = kNonPirateMission;
	Specs->fNumShips = MissSpecOneShip | MissSpecTwoShip;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 10;
	Specs->fMaxDate = 30;
	Specs->fWait = 99;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 125, 75, 25, -25);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kEnemyTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kAlliedTeam, info.fAlliedRace);
	fMission.mSetTeamRace (kEnemyReinforcementTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kAlliedReinforcementTeam, info.fAlliedRace);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyTeam, kAlliedTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyReinforcementTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyReinforcementTeam, kAlliedTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyReinforcementTeam, kAlliedReinforcementTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kEnemyReinforcementTeam, kEnemyTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kAlliedTeam, kPlayerTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kAlliedReinforcementTeam, kPlayerTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kAlliedReinforcementTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kAlliedReinforcementTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

	// only one map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	Ship.fShipPosition = kStartPosition_G;
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* EnemyReinforcementTeam = fMission.mGetTeamHandle( kEnemyReinforcementTeam );
	tTeamInfo* AlliedReinforcementTeam = fMission.mGetTeamHandle( kAlliedReinforcementTeam );

	int32 DiffLevel = fMission.mGetDifficultyLevel();
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * ( .6 + ( DiffLevel * .2 ));
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * ( .9 + ( DiffLevel * .3 ));
	int32 RandomBPV = fMission.mRandomInt32( LowBPV, HighBPV );
	int32 TotalShips = PlayerTeam->mGetShipsInPlayCount();
	int32 OldX = 0;
	int32 OldY = 0;
	int32 NewX = 0;
	int32 NewY = 0;

	// Create the enemy ships and the allied ship at the beginning of the mission
	if( Mode == 0 )
	{
		// Enemy Team
		EnemyTeam->mCreateFleet( TotalShips, HighBPV, kClassFrigate, kClassHeavyCruiser, 0, kStartPosition_W | kGroupAround );
		EnemyTeam->mCreateShip( RandomBPV, kClassLightCruiser, kClassHeavyBattlecruiser, 50, kStartPosition_Z );

		// Allied Team
		AlliedReinforcementTeam->mCreateShip( HighBPV, kClassFrigate, kClassHeavyCruiser, 0, kStartPosition_T );
		AlliedTeam->mCreateShip( kMaxBPV, kClassFreighter, 40, kStartPosition_U );
	}

	// Enemy Reinforcements
	if( Mode == 1 )
	{
		fPlayerShip->mGetGridPosition( &OldX, &OldY );
		NewX = OldX + fMission.mRandomInt32( 14, 18 );
		NewY = OldY + fMission.mRandomInt32( -2, 2 );
		EnemyReinforcementTeam->mCreateShip( HighBPV, kClassFrigate, kClassHeavyBattlecruiser, 0, kStartPosition_Specified, NewX, NewY );
		EnemyReinforcementTeam->mCreateShip( LowBPV, kClassFrigate, kClassLightCruiser, 0, kStartPosition_Specified, NewX, NewY );
		EnemyReinforcementTeam->mCreateShip( RandomBPV, kClassLightCruiser, kClassHeavyBattlecruiser, 50, kStartPosition_Specified, NewX, NewY );
	}

	// Allied Reinforcements
	if( Mode == 2 )
	{
		AlliedReinforcementTeam->mCreateShip( kMaxBPV, kClassFrigate, kClassFrigate, 1, kStartPosition_J );
		AlliedReinforcementTeam->mCreateShip( kMaxBPV, kClassLightCruiser, kClassLightCruiser, 1, kStartPosition_K );
		AlliedReinforcementTeam->mCreateShip( kMaxBPV, kClassLightCruiser, kClassLightCruiser, 100, kStartPosition_L );
		AlliedReinforcementTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, kClassHeavyCruiser, 1, kStartPosition_M );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* EnemyReinforcementTeam = fMission.mGetTeamHandle( kEnemyReinforcementTeam );
	tTeamInfo* AlliedReinforcementTeam = fMission.mGetTeamHandle( kAlliedReinforcementTeam );
	tShipInfo* Ship;

	int32 AllyX = 0;
	int32 AllyY = 0;

	fMission.mGetGridPosition( 'M', &AllyX, &AllyY );
	
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	
	for( Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip() )
	{
		Ship->mSetShipThrust( 10 );
		Ship->mTrackTacticalDistance( 50.0, AllyX, AllyY );	// distance to allied reinforcements
	}

	for( Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip() )
	{
		Ship->mSetCommButton( 1, kButtonHail_text );
	}

	// Start the initial timer
	fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* EnemyReinforcementTeam = fMission.mGetTeamHandle( kEnemyReinforcementTeam );
	tTeamInfo* AlliedReinforcementTeam = fMission.mGetTeamHandle( kAlliedReinforcementTeam );

	if( Ship->mGetTeam() == kEnemyReinforcementTeam )
	{
		for( Ship = EnemyReinforcementTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyReinforcementTeam->mGetNextShip() )
		{
			Ship->mSetCommButton( 1, kButtonHail_text );
		}
	}
	else
	{
		for( Ship = AlliedReinforcementTeam->mGetFirstShip(); Ship != NULL; Ship = AlliedReinforcementTeam->mGetNextShip() )
		{
			Ship->mSetCommButton( 1, kButtonHail_text );
		}
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* EnemyReinforcementTeam = fMission.mGetTeamHandle( kEnemyReinforcementTeam );
	tTeamInfo* AlliedReinforcementTeam = fMission.mGetTeamHandle( kAlliedReinforcementTeam );

	bool EnemiesAlive = false;

	if( EnemyTeam->mGetShipsInPlayCount() > 0 )
	{
		EnemiesAlive = true;
	}

	if( EnemyReinforcementTeam->mGetShipsInPlayCount() > 0 )
	{
		EnemiesAlive = true;
	}

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kEnemyTeam, kAlliedTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kEnemyReinforcementTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kEnemyReinforcementTeam, kAlliedTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kEnemyReinforcementTeam, kAlliedReinforcementTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kEnemyReinforcementTeam, kEnemyTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kAlliedTeam, kPlayerTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kAlliedReinforcementTeam, kPlayerTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kAlliedReinforcementTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kAlliedReinforcementTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					break;

				case kTimerTurnsSet:
					if( MissionAct == 0 )
					{
						MissionAct++;
						fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage );
						fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
					}
					else if( MissionAct == 1 )
					{
						MissionAct++;

						Ship = AlliedTeam->mGetFirstShip();
						Ship->mSetCommButton( 1, kButtonHail_text );
						fMission.mDisplayMessage( kWarning_msg, kMissionScriptMessage, Ship );

						for( Ship = AlliedReinforcementTeam->mGetFirstShip(); Ship != NULL; Ship = AlliedTeam->mGetNextShip() )
						{
							Ship->mSetCommButton( 1, kButtonHail_text );
						}

						fMission.mSetTimedInterval( kTimerTurnsSet, 10 );
					}
					else if( MissionAct == 2 )
					{
						MissionAct++;
						fMission.mDisplayMessage( kEnemyApproaching_msg, kMissionScriptMessage );
						mCreateMissionShips( 1 );	// create the enemy reinforcements

						for( Ship = AlliedTeam->mGetFirstShip(); Ship != NULL; Ship = AlliedTeam->mGetNextShip() )
						{
							Ship->mClearCommButton( 1 );
						}
					}
					break;
			}
			break;
		
		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kAlliedTeam )
			{
				fMission.mDisplayMessage( kDiplomatDead_msg, kMissionScriptMessage );
				DiplomatDead = true;
			}
			
			if( EnemiesAlive == false )
			{
				return kPlayerWins;
			}

			if( Ship->mGetTeam() == kPlayerTeam )
			{
				Losses++;

				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					PlayerDead = true;
					return kPlayerLoses;
				}
			}
			break;

		case kNotifyOnCapture:
			if(( Ship->mGetTeam() == kEnemyTeam ) || ( Ship->mGetTeam() == kEnemyReinforcementTeam ))
			{
				EnemyCaptured++;
			}
			break;

		case kNotifyCommButtonPress:
			if( Ship->mGetTeam() == kAlliedTeam )
			{
				switch( fEventID )
				{
					case kButtonHail_text:
						fMission.mDisplayMessage( kAllyResponse_msg, kFleetScriptMessage, Ship );
						Ship->mSetCommButton( 1, kButtonAcknowledge_text );
						break;

					case kButtonAcknowledge_text:
						fMission.mDisplayMessage( kAllyResponse2_msg, kFleetScriptMessage, Ship );
						break;
				}
			}
			else if ((Ship->mGetTeam() == kEnemyTeam) || (Ship->mGetTeam() == kEnemyReinforcementTeam))
			{
				fMission.mDisplayMessage( kEnemyNoResponse_msg, kFleetScriptMessage );
			}
			else if( Ship->mGetTeam() == kAlliedReinforcementTeam )
			{
				if( DiplomatDead == false )
				{
					fMission.mDisplayMessage( kAlliedReinforceAlive_msg, kFleetScriptMessage, Ship );
				}
				else
				{
					fMission.mDisplayMessage( kAlliedReinforceDead_msg, kFleetScriptMessage, Ship );
				}
			}
			break;

		case kNotifyAtDistance:
			if( Reinforced == false )
			{
				Reinforced = true;
				fMission.mDisplayMessage( kAllyHelping_msg, kMissionScriptMessage );
				mCreateMissionShips( 2 );
			}
			break;

		case kNotifyDisengaged:
			if(( Ship->mGetTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				DiplomatDead = true;
				return kPlayerLoses;
			}
			break;

		case kNotifyPlayerLeaving:
			if( EnemiesAlive == true )
			{
				DiplomatDead = true;
				return kPlayerLoses;
			}
			break;
	}
	
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	if( PlayerDead == true )
	{
		return( kDevastatingDefeat );
	}

	if( DiplomatDead == true )
	{
		return( kDefeat );
	}

	if( EnemyCaptured > 0 )
	{
		return( kAstoundingVictory );
	}

	if( Losses < 1 )
	{
		return( kVictory );
	}

	return( kDraw );
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList( VictoryLevel + kAstoundingVictory_msg );

	if( EnemyCaptured > 0 )
	{
		fMission.mAddToDebriefingList( kEnemyCaptured_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
