#include "scriptinterface.h"				// must have this as the first include

#include "Gen_TemplateText.h"					// must have these two includes
#include "Gen_TemplateMaps.h"

// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
//
// To insert a new script project into the workspace:
//		1. create a new folder off of the \SFCScripts folder.
//		2. copy template files (.cpp and .h) into new folder.
//		3. rename copied template files the same name as new folder.
//		4. start MSVC++, load SFCScripts workspace.
//		5. click Project on main menu, select Add to Project->New.
//		6. click Project on New menu, select Win32 Dynamic-Link Library.
//		7. click Add to Current Project.
//		8. click Dependency of _RebuildAllScripts.
//		9. enter Project Name (same as new folder name), click OK.
//		10. select Empty DLL Project, click Finish. Click OK to confirm.
//		11. right click on new project in workspace window.
//		12. select Add Files to Project.
//		13. highlight all files in new folder (.cpp and .h). Click OK.
//		14. click Project on main menu, select Dependencies.
//		15. set new project dependent on _ScriptInterface. Click OK.
//		16. click Project on main menu, select Settings...
//		17. select new project and set Settings For to Release.
//		18. click C/C++ tab, select Preprocessor category.
//		19. add to Addition Include Dir: ..\scriptinterface,..\includes
//		20. click Link tab, select General category.
//		21. in Output File Name box change .dll to .scr.
//		22. add to Object/Library Modules box: scriptinterface.lib StarFleet1.lib StarFleet2.lib
//		23. select Input category.
//		24. add to Ignore Libraries box: libcd.lib,libcmt.lib
//		25. add to Additional Library Path: ..\libs,..\,..\scriptinterface
//		26. open new .cpp file and change text and map include file names.
//		27. select Build->Set Active Configuration..., set to Release.
//		28. Compile.
//		29. copy .scr file from \Release folder into \Assets\Scripts folder of SFC.
//
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-human player teams in the script
// (use kPlayerTeam to access the current human players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for human player teams.
	kReserved = kTeam6,

	// add all non-human player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam,
};

// -----------------------------------------------------------------------------------
// this area can be used to add extra enums, work buffers or whatever is needed

// -----------------------------------------------------------------------------------

//---------------------------------------------------------------------------------------------
//			START OF SCRIPT ROUTINES
//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaign list
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;				// use values from eMissionTypes enum in missionspec.h
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;					// use values from eMissSpecSect enum in missionspec.h
	Specs->fPlayerRace = kAllRaceMission;				// use values from eMissSpecRace enum in missionspec.h
	Specs->fEnemyRace = kAllRaceMission;				// use values from eMissSpecRace enum in missionspec.h
	Specs->fNumShips = MissSpecAllShips;				// use values from Bit masks in missionspec.h
	Specs->fMinBPV = 0;									// player must have at least this much BPV to play this mission
	Specs->fMaxBPV = kMaxBPV;							// player cannot have more than this much BPV to play this mission
	Specs->fMinDate = 0;								// player can play this mission starting on this year date (date starts at 0
														//   and advances 1 year at a time)
	Specs->fMaxDate = 9999;								// player will not be able to play this mission after this year date
	Specs->fWait = 0;									// has multiple uses depending on which fMissionType was specified above:
														// for kCampaignMission:
														//		specifies how many years Dynaverse will wait before selecting 
														//		this misison again. when fWait == kUniqueMission then this mission
														//		is played only once per campaign.
														// for kTutorialMission & kTechnicalMission:
														//		specifies this missions position in the academy menu. must be 0 - 5.
														// for kHistoricalMission & kMultiplayerMission:
														//		unused - should be 0
	Specs->fEnableSupplyDock = kEnableSupplyDock;		// enables/disables the Supply dock button in the briefing screen.
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (60, 40, 20, 0, -5);

	// can change the race of enemy or ally if needed (but NOT player)

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kEnemyTeam, info.fEnemyRace);

	// must setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission.
// can also change the default starting position of the ship (Ship.fShipPosition).
// all players ships start at kStartPosition_G and increament from there.
// (you are not able to specify an X/Y grid location)
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before the mission starts with Mode == 0.
// also can be called after the mission starts (Mode can be any user supplied value (expect 0)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	// create enemy ship(s)
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	EnemyTeam->mCreateFleet (PlayerTeam->mGetShipsInPlayCount(),
							 PlayerTeam->mGetCombatBPV(), 
							 kClassFrigate,
							 PlayerTeam->mGetMaxClassType(),
							 50,
							 kStartPosition_U);
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return index to briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tShipInfo* Ship;
	tShipInfo* Target;

	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	Target = EnemyTeam->mGetFirstShip();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipThrust (10);
		Ship->mSetShipTarget (Target, kHighPriority);
		Target = EnemyTeam->mGetNextShip();
	}

	// configure enemy ship(s)
	Target = PlayerTeam->mGetFirstShip();
	for (Ship = EnemyTeam->mGetFirstShip (); Ship != NULL; Ship = EnemyTeam->mGetNextShip ())
	{
		Ship->mSetShipTarget (Target, kHighPriority);
		Target = PlayerTeam->mGetNextShip();
	}
}

//---------------------------------------------------------------------------------------------
// should return whatever team is considered to be the primary enemy.
// NOTE: should only return values found in eTeamTypes enum above
int32 tScript::mGetEnemyTeamID (void)
{
	return kEnemyTeam;
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during the mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	switch (Event)
	{
		case kNotifyAtDeath:
			break;
		case kNotifyOnCapture:
			break;
		case kNotifyDisengaged:
			break;
		case kNotifyAtDistance:
			break;
		case kNotifyAtDestination:
			break;
		case kNotifyAtTimedInterval:
			break;
		case kNotifyCommButtonPress:
			break;
		case kNotifyDamage:
			break;
		case kNotifyDeepScan:
			break;
		case kNotifyAlertStatusChange:
			break;
		case kNotifyProbeLaunch:
			break;
		case kNotifyBeginOrbit:
			break;
		case kNotifyEndOrbit:
			break;
		case kNotifyBeginTractor:
			break;
		case kNotifyEndTractor:
			break;
		case kNotifyFlashingButtonPressed:
			break;
		case kNotifyTransportedItemTo:
			break;
		case kNotifyTransportedItemFrom:
			break;
		case kNotifyChangingShip:
			break;
		case kNotifySpecialTransport:
			break;
		case kNotifyPlayerLeaving:
			break;
		case kNotifyHostChanged:
			break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);

	// determine who gets the combat BPV
	int32 PlayerBPV = PlayerTeam->mGetCombatBPV();
	int32 EnemyBPV = EnemyTeam->mGetCombatBPV();
	if (PlayerBPV > EnemyBPV)
	{
		EnemyBPV = PlayerBPV - EnemyBPV;
		PlayerBPV = 0;
	}
	else
	{
		PlayerBPV = EnemyBPV - PlayerBPV;
		EnemyBPV = 0;
	}

	// add in assesed damage from the opposing team
	PlayerBPV += EnemyTeam->mAssessDamages(PlayerTeam);
	EnemyBPV += PlayerTeam->mAssessDamages(EnemyTeam);

	return (fMission.mGetVictoryLevel (PlayerBPV, EnemyBPV));
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

