#include "scriptinterface.h"				// must have this as the first include

#include "Mul_TemplateText.h"					// must have these two includes
#include "Mul_TemplateMaps.h"

// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
//
// To insert a new script project into the workspace:
//		1. create a new folder off of the \SFCScripts folder.
//		2. copy template files (.cpp and .h) into new folder.
//		3. rename copied template files the same name as new folder.
//		4. start MSVC++, load SFCScripts workspace.
//		5. click Project on main menu, select Add to Project->New.
//		6. click Project on New menu, select Win32 Dynamic-Link Library.
//		7. click Add to Current Project.
//		8. click Dependency of _RebuildAllScripts.
//		9. enter Project Name (same as new folder name), click OK.
//		10. select Empty DLL Project, click Finish. Click OK to confirm.
//		11. right click on new project in workspace window.
//		12. select Add Files to Project.
//		13. highlight all files in new folder (.cpp and .h). Click OK.
//		14. click Project on main menu, select Dependencies.
//		15. set new project dependent on _ScriptInterface. Click OK.
//		16. click Project on main menu, select Settings...
//		17. select new project and set Settings For to Release.
//		18. click C/C++ tab, select Preprocessor category.
//		19. add to Addition Include Dir: ..\scriptinterface,..\includes
//		20. click Link tab, select General category.
//		21. in Output File Name box change .dll to .scr.
//		22. add to Object/Library Modules box: scriptinterface.lib StarFleet1.lib StarFleet2.lib
//		23. select Input category.
//		24. add to Ignore Libraries box: libcd.lib,libcmt.lib
//		25. add to Additional Library Path: ..\libs,..\,..\scriptinterface
//		26. open new .cpp file and change text and map include file names.
//		27. select Build->Set Active Configuration..., set to Release.
//		28. Compile.
//		29. copy .scr file from \Release folder into \Assets\Scripts folder of SFC.
//
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-human player teams in the script
// (use kPlayerTeam to access the current human players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for human player teams.
	kReserved = kTeam6,

	// add all non-human player teams (enemy, allied, monsters, etc...) after this point.
};

// -----------------------------------------------------------------------------------
// this area can be used to add extra enums, work buffers or whatever is needed

#include "teamtracker.h"
tTeamTrackerClass fTeamTracker;

// -----------------------------------------------------------------------------------

//---------------------------------------------------------------------------------------------
//			START OF SCRIPT ROUTINES
//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaign list
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kMultiplayerMission;			// use values from eMissionTypes enum in missionspec.h
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;					// not used by multiplayer missions
	Specs->fPlayerRace = kAllRaceMission;				// not used by multiplayer missions
	Specs->fEnemyRace = kAllRaceMission;				// not used by multiplayer missions
	Specs->fMinBPV = 0;									// not used by multiplayer missions
	Specs->fMaxBPV = kMaxBPV;							// not used by multiplayer missions
	Specs->fMinDate = 9999;								// not used by multiplayer missions
	Specs->fMaxDate = 9999;								// not used by multiplayer missions

	// multiplayer menu specific settings:
	//		max players (fWait)
	//		max ships (fNumShips)
	//		enable supply dock
	//		disable teams
	//		disable AI
	Specs->fNumShips = MissSpecAllShips;			// max number of ships per player
	Specs->fWait = 6;								// max number of players
	Specs->fEnableSupplyDock = kEnableSupplyDock;	// enable supply dock button in multiplayer menu

}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	// must setup the races and relations of the reserved teams in the script
	fMission.mSetMultiplayerTeams (info);

	// setup the team tracker class
	fTeamTracker.mInitialize (kPlayerTeam, info, 0);

	// must setup races of non-menu teams (if any)

	// must setup team relations and target goals of non-menu teams (if any)

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission.
// can also change the default starting position of the ship (Ship.fShipPosition).
// all players ships start at kStartPosition_G and increament from there.
// (you are not able to specify an X/Y grid location)
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	Ship.fShipPosition |= kGroupAround;
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before the mission starts with Mode == 0.
// also can be called after the mission starts (Mode can be any user supplied value (expect 0)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	// only the host needs to create the AI ships
	if (fThisScriptIsTheHost)
	{
		tTeamInfo* Team;
		int32 position;

		// create friendly AI ships first
		for (int32 i = 0; i < fMission.fTotalAlliedCPUTeams; i++)
		{
			Team = fMission.mGetTeamHandle (kCPU_AlliedTeams, i);
			position = Team->mGetMultiplayerTeam() + kStartPosition_G;
			Team->mCreateCPUFleet (20, position | kGroupAround);
		}

		// create enemy AI ships last
		for (i = 0; i < fMission.fTotalEnemyCPUTeams; i++)
		{
			Team = fMission.mGetTeamHandle (kCPU_EnemyTeams, i);
			position = Team->mGetMultiplayerTeam() + kStartPosition_G;
			Team->mCreateCPUFleet (20, position | kGroupAround);
		}
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return index to briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	// only the host needs to setup the AI ships
	if (fThisScriptIsTheHost)
	{
	}
}

//---------------------------------------------------------------------------------------------
// should return whatever team is considered to be the primary enemy.
// NOTE: should only return values found in eTeamTypes enum above
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during the mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	switch (Event)
	{
		case kNotifyAtDeath:
			break;
		case kNotifyOnCapture:
			break;
		case kNotifyDisengaged:
			break;
		case kNotifyAtDistance:
			break;
		case kNotifyAtDestination:
			break;
		case kNotifyAtTimedInterval:
			break;
		case kNotifyCommButtonPress:
			break;
		case kNotifyDamage:
			break;
		case kNotifyDeepScan:
			break;
		case kNotifyAlertStatusChange:
			break;
		case kNotifyProbeLaunch:
			break;
		case kNotifyBeginOrbit:
			break;
		case kNotifyEndOrbit:
			break;
		case kNotifyBeginTractor:
			break;
		case kNotifyEndTractor:
			break;
		case kNotifyFlashingButtonPressed:
			break;
		case kNotifyTransportedItemTo:
			break;
		case kNotifyTransportedItemFrom:
			break;
		case kNotifyChangingShip:
			break;
		case kNotifySpecialTransport:
			break;
		case kNotifyPlayerLeaving:
			fTeamTracker.mTeamGone (eTeamID(fEventID), NULL);
			break;
		case kNotifyHostChanged:
			fThisScriptIsTheHost = kTrue;
			break;
	}

	// multiplayer missions need to change kMissionFailed, kPlayerDestroyed, kPlayerCaptured
	// to kPlayerLoses in order to get the debriefing messages to show up - TeamTracker handles that task
	return fTeamTracker.mGetTeamsStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	if (fTeamTracker.mGetTeamsStatus() == kPlayerWins)
	{
		return kAstoundingVictory;
	}

	return kDevastatingDefeat;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
